package cs2.introml.tests;

import java.io.IOException;

import cs2.introml.Classifier;
import cs2.introml.DataSet;
import cs2.introml.DenseExample;
import cs2.introml.DistanceFunction;
import cs2.introml.EuclideanDistanceSquared;
import cs2.introml.LibSVMDataSet;
import cs2.introml.NearestNeighborClassifier;
import cs2.introml.SparseExample;
import cs2.introml.TieBreaker;
import cs2.introml.ZeroRClassifier;
import junit.framework.TestCase;

public class P1Tests extends TestCase {
	
	public void testTie() {
		
		final TieBreaker<String> tb = new TieBreaker<>( 1 );
		
		tb.candidate( "a", 1 );
		assertTrue( tb.getWinner().equals( "a" ) );
		
		tb.candidate( "b", 2 );
		assertTrue( tb.getWinner().equals( "a" ) );
		
		tb.candidate( "alpha", 1 );
		assertTrue( tb.getWinner().equals( "a" ) );
		
		tb.candidate( "aleph", 1 );
		assertTrue( tb.getWinner().equals( "a" ) );
		
		tb.candidate( "uno", 1 );
		assertTrue( tb.getWinner().equals( "uno" ) );
		
	}
	
	public void testZeroRGolf() throws IOException {
		
		final DataSet ds = new LibSVMDataSet( P1Tests.class.getResourceAsStream( "data/golf.data" ), 4, LibSVMDataSet.StringInterpreter );
		final Classifier c = new ZeroRClassifier( 42 );
		assertEquals( 0.64, c.evaluate( ds, ds ), 0.01 );
		
	}
	
	public void testNNGolf() throws IOException {
		
		final DataSet ds = new LibSVMDataSet( P1Tests.class.getResourceAsStream( "data/golf.data" ), 4, LibSVMDataSet.StringInterpreter );
		final Classifier c = new NearestNeighborClassifier( new EuclideanDistanceSquared(), 42 );
		assertEquals( 1.0, c.evaluate( ds, ds ), 0.01 );
		
	}
	
	public void testDistance() {
		
		// [ 1, 2, 3 ]
		final DenseExample deFoo = new DenseExample( "foo", new double[] { 1., 2., 3. } );
		
		// [ 0, 6, 0 ]
		final SparseExample seBar = new SparseExample( "bar", 3 );
		seBar.setFeature( 1, 6 );
		
		final DistanceFunction df = new EuclideanDistanceSquared();
		
		assertEquals( 0., df.distance( deFoo, deFoo ) );
		assertEquals( 0., df.distance( seBar, seBar ) );
		
		assertEquals( ( ( 1.*1. ) + ( 4.*4. ) + ( 3.*3. ) ), df.distance( deFoo, seBar ) );
		assertEquals( ( ( 1.*1. ) + ( 4.*4. ) + ( 3.*3. ) ), df.distance( seBar, deFoo ) );
		
	}

}
